@@ -6,12 +6,21 @@
 #ifndef SCINTILLAGTK_H
 #define SCINTILLAGTK_H
 
-namespace Scintilla {
+namespace Scintilla::Internal {
 
 class ScintillaGTKAccessible;
 
 #define OBJECT_CLASS GObjectClass
 
+struct FontOptions {
+	cairo_antialias_t antialias {};
+	cairo_subpixel_order_t order {};
+	cairo_hint_style_t hint {};
+	FontOptions() noexcept = default;
+	explicit FontOptions(GtkWidget *widget) noexcept;
+	bool operator==(const FontOptions &other) const noexcept;
+};
+
 class ScintillaGTK : public ScintillaBase {
 	friend class ScintillaGTKAccessible;
 
@@ -21,7 +30,6 @@ class ScintillaGTK : public ScintillaBase {
 	Window scrollbarh;
 	GtkAdjustment *adjustmentv;
 	GtkAdjustment *adjustmenth;
-	Window wSelection;
 	int verticalScrollBarWidth;
 	int horizontalScrollBarHeight;
 
@@ -36,20 +44,26 @@ class ScintillaGTK : public ScintillaBase {
 
 	GtkWidgetClass *parentClass;
 
-	static GdkAtom atomUTF8;
-	static GdkAtom atomString;
-	static GdkAtom atomUriList;
-	static GdkAtom atomDROPFILES_DND;
+	static inline GdkAtom atomUTF8 {};
+	static inline GdkAtom atomUTF8Mime {};
+	static inline GdkAtom atomString {};
+	static inline GdkAtom atomUriList {};
+	static inline GdkAtom atomDROPFILES_DND {};
 	GdkAtom atomSought;
+	size_t inClearSelection = 0;
 
 #if PLAT_GTK_WIN32
 	CLIPFORMAT cfColumnSelect;
 #endif
 
+	bool preeditInitialized;
 	Window wPreedit;
 	Window wPreeditDraw;
 	GtkIMContext *im_context;
-	PangoScript lastNonCommonScript;
+	GUnicodeScript lastNonCommonScript;
+
+	GtkSettings *settings;
+	gulong settingsHandlerId;
 
 	// Wheel mouse support
 	unsigned int linesPerScroll;
@@ -67,6 +81,7 @@ class ScintillaGTK : public ScintillaBase {
 	bool repaintFullWindow;
 
 	guint styleIdleID;
+	FontOptions fontOptionsPrevious;
 	int accessibilityEnabled;
 	AtkObject *accessible;
 
@@ -78,7 +93,7 @@ public:
 	ScintillaGTK &operator=(const ScintillaGTK &) = delete;
 	ScintillaGTK &operator=(ScintillaGTK &&) = delete;
 	virtual ~ScintillaGTK();
-	static ScintillaGTK *FromWidget(GtkWidget *widget);
+	static ScintillaGTK *FromWidget(GtkWidget *widget) noexcept;
 	static void ClassInit(OBJECT_CLASS *object_class, GtkWidgetClass *widget_class, GtkContainerClass *container_class);
 private:
 	void Init();
@@ -90,17 +105,19 @@ private:
 	Sci::Position TargetAsUTF8(char *text) const;
 	Sci::Position EncodedFromUTF8(const char *utf8, char *encoded) const;
 	bool ValidCodePage(int codePage) const override;
+	std::string UTF8FromEncoded(std::string_view encoded) const override;
+	std::string EncodedFromUTF8(std::string_view utf8) const override;
 public: 	// Public for scintilla_send_message
-	sptr_t WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam) override;
+	sptr_t WndProc(Scintilla::Message iMessage, Scintilla::uptr_t wParam, Scintilla::sptr_t lParam) override;
 private:
-	sptr_t DefWndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam) override;
+	sptr_t DefWndProc(Scintilla::Message iMessage, Scintilla::uptr_t wParam, Scintilla::sptr_t lParam) override;
 	struct TimeThunk {
 		TickReason reason;
 		ScintillaGTK *scintilla;
 		guint timer;
-		TimeThunk() noexcept : reason(tickCaret), scintilla(nullptr), timer(0) {}
+		TimeThunk() noexcept : reason(TickReason::caret), scintilla(nullptr), timer(0) {}
 	};
-	TimeThunk timers[tickDwell+1];
+	TimeThunk timers[static_cast<size_t>(TickReason::dwell)+1];
 	bool FineTickerRunning(TickReason reason) override;
 	void FineTickerStart(TickReason reason, int millis, int tolerance) override;
 	void FineTickerCancel(TickReason reason) override;
@@ -117,13 +134,13 @@ private:
 	void ReconfigureScrollBars() override;
 	void NotifyChange() override;
 	void NotifyFocus(bool focus) override;
-	void NotifyParent(SCNotification scn) override;
-	void NotifyKey(int key, int modifiers);
+	void NotifyParent(Scintilla::NotificationData scn) override;
+	void NotifyKey(Scintilla::Keys key, Scintilla::KeyMod modifiers);
 	void NotifyURIDropped(const char *list);
 	const char *CharacterSetID() const;
-	CaseFolder *CaseFolderForEncoding() override;
-	std::string CaseMapString(const std::string &s, int caseMapping) override;
-	int KeyDefault(int key, int modifiers) override;
+	std::unique_ptr<CaseFolder> CaseFolderForEncoding() override;
+	std::string CaseMapString(const std::string &s, CaseMapping caseMapping) override;
+	int KeyDefault(Scintilla::Keys key, Scintilla::KeyMod modifiers) override;
 	void CopyToClipboard(const SelectionText &selectedText) override;
 	void Copy() override;
 	void RequestSelection(GdkAtom atomSelection);
@@ -132,11 +149,12 @@ private:
 	void AddToPopUp(const char *label, int cmd = 0, bool enabled = true) override;
 	bool OwnPrimarySelection();
 	void ClaimSelection() override;
+	static bool IsStringAtom(GdkAtom type);
 	void GetGtkSelectionText(GtkSelectionData *selectionData, SelectionText &selText);
-	void InsertSelection(GtkSelectionData *selectionData);
+	void InsertSelection(GtkClipboard *clipBoard, GtkSelectionData *selectionData);
 public:	// Public for SelectionReceiver
 	GObject *MainObject() const noexcept;
-	void ReceivedClipboard(GtkSelectionData *selection_data) noexcept;
+	void ReceivedClipboard(GtkClipboard *clipBoard, GtkSelectionData *selection_data) noexcept;
 private:
 	void ReceivedSelection(GtkSelectionData *selection_data);
 	void ReceivedDrop(GtkSelectionData *selection_data);
@@ -145,9 +163,13 @@ private:
 	static void ClipboardGetSelection(GtkClipboard *clip, GtkSelectionData *selection_data, guint info, void *data);
 	static void ClipboardClearSelection(GtkClipboard *clip, void *data);
 
+	void ClearPrimarySelection();
+	void PrimaryGetSelectionThis(GtkClipboard *clip, GtkSelectionData *selection_data, guint info);
+	static void PrimaryGetSelection(GtkClipboard *clip, GtkSelectionData *selection_data, guint info, gpointer pSci);
+	void PrimaryClearSelectionThis(GtkClipboard *clip);
+	static void PrimaryClearSelection(GtkClipboard *clip, gpointer pSci);
+
 	void UnclaimSelection(GdkEventSelection *selection_event);
-	static void PrimarySelection(GtkWidget *widget, GtkSelectionData *selection_data, guint info, guint time_stamp, ScintillaGTK *sciThis);
-	static gboolean PrimaryClear(GtkWidget *widget, GdkEventSelection *event, ScintillaGTK *sciThis);
 	void Resize(int width, int height);
 
 	// Callback functions
@@ -169,6 +191,7 @@ private:
 	static void GetPreferredHeight(GtkWidget *widget, gint *minimalHeight, gint *naturalHeight);
 #endif
 	static void SizeAllocate(GtkWidget *widget, GtkAllocation *allocation);
+	void CheckForFontOptionChange();
 #if GTK_CHECK_VERSION(3,0,0)
 	gboolean DrawTextThis(cairo_t *cr);
 	static gboolean DrawText(GtkWidget *widget, cairo_t *cr, ScintillaGTK *sciThis);
@@ -209,8 +232,8 @@ private:
 	void PreeditChangedInlineThis();
 	void PreeditChangedWindowedThis();
 	static void PreeditChanged(GtkIMContext *context, ScintillaGTK *sciThis);
-	void MoveImeCarets(int pos);
-	void DrawImeIndicator(int indicator, int len);
+	void MoveImeCarets(Sci::Position pos);
+	void DrawImeIndicator(int indicator, Sci::Position len);
 	void SetCandidateWindowPos();
 
 	static void StyleSetText(GtkWidget *widget, GtkStyle *previous, void *);
@@ -238,7 +261,7 @@ private:
 	static gboolean IdleCallback(gpointer pSci);
 	static gboolean StyleIdle(gpointer pSci);
 	void IdleWork() override;
-	void QueueIdleWork(WorkNeeded::workItems items, Sci::Position upTo) override;
+	void QueueIdleWork(WorkItems items, Sci::Position upTo) override;
 	void SetDocPointer(Document *document) override;
 	static void PopUpCB(GtkMenuItem *menuItem, ScintillaGTK *sciThis);
 
@@ -251,6 +274,8 @@ private:
 
 	static sptr_t DirectFunction(sptr_t ptr,
 				     unsigned int iMessage, uptr_t wParam, sptr_t lParam);
+	static sptr_t DirectStatusFunction(sptr_t ptr,
+				     unsigned int iMessage, uptr_t wParam, sptr_t lParam, int *pStatus);
 };
 
 // helper class to watch a GObject lifetime and get notified when it dies
