@@ -6,7 +6,7 @@
 // Author: Jason Haslam
 //
 // Additions Copyright (c) 2011 Archaeopteryx Software, Inc. d/b/a Wingware
-// ScintillaQt.cpp - Qt specific subclass of ScintillaBase
+// @file ScintillaQt.cpp - Qt specific subclass of ScintillaBase
 
 #include "ScintillaQt.h"
 #include "PlatQt.h"
@@ -23,7 +23,7 @@
 #include <QTimer>
 
 using namespace Scintilla;
-
+using namespace Scintilla::Internal;
 
 ScintillaQt::ScintillaQt(QAbstractScrollArea *parent)
 : QObject(parent), scrollArea(parent), vMax(0),  hMax(0), vPage(0), hPage(0),
@@ -32,7 +32,7 @@ ScintillaQt::ScintillaQt(QAbstractScrollArea *parent)
 
 	wMain = scrollArea->viewport();
 
-	imeInteraction = imeInline;
+	imeInteraction = IMEInteraction::Inline;
 
 	// On OS X drawing text into a pixmap moves it around 1 pixel to
 	// the right compared to drawing it directly onto a window.
@@ -41,9 +41,7 @@ ScintillaQt::ScintillaQt(QAbstractScrollArea *parent)
 
 	Init();
 
-	for (TickReason tr = tickCaret; tr <= tickDwell; tr = static_cast<TickReason>(tr + 1)) {
-		timers[tr] = 0;
-	}
+	std::fill(timers, std::end(timers), 0);
 }
 
 ScintillaQt::~ScintillaQt()
@@ -61,6 +59,8 @@ void ScintillaQt::execCommand(QAction *action)
 #if defined(Q_OS_WIN)
 static const QString sMSDEVColumnSelect("MSDEVColumnSelect");
 static const QString sWrappedMSDEVColumnSelect("application/x-qt-windows-mime;value=\"MSDEVColumnSelect\"");
+static const QString sVSEditorLineCutCopy("VisualStudioEditorOperationsLineCutCopyClipboardTag");
+static const QString sWrappedVSEditorLineCutCopy("application/x-qt-windows-mime;value=\"VisualStudioEditorOperationsLineCutCopyClipboardTag\"");
 #elif defined(Q_OS_MAC)
 static const QString sScintillaRecPboardType("com.scintilla.utf16-plain-text.rectangular");
 static const QString sScintillaRecMimeType("text/x-scintilla.utf16-plain-text.rectangular");
@@ -171,7 +171,7 @@ static QString StringFromSelectedText(const SelectionText &selectedText)
 	}
 }
 
-static void AddRectangularToMime(QMimeData *mimeData, QString su)
+static void AddRectangularToMime(QMimeData *mimeData, [[maybe_unused]] QString su)
 {
 #if defined(Q_OS_WIN)
 	// Add an empty marker
@@ -182,13 +182,20 @@ static void AddRectangularToMime(QMimeData *mimeData, QString su)
 	// clipboard format is supposed to be UTF-16, not UTF-8.
 	mimeData->setData(sScintillaRecMimeType, su.toUtf8());
 #else
-	Q_UNUSED(su);
 	// Linux
 	// Add an empty marker
 	mimeData->setData(sMimeRectangularMarker, QByteArray());
 #endif
 }
 
+static void AddLineCutCopyToMime([[maybe_unused]] QMimeData *mimeData)
+{
+#if defined(Q_OS_WIN)
+	// Add an empty marker
+	mimeData->setData(sVSEditorLineCutCopy, QByteArray());
+#endif
+}
+
 static bool IsRectangularInMime(const QMimeData *mimeData)
 {
 	QStringList formats = mimeData->formats();
@@ -213,6 +220,23 @@ static bool IsRectangularInMime(const QMimeData *mimeData)
 	return false;
 }
 
+static bool IsLineCutCopyInMime(const QMimeData *mimeData)
+{
+	QStringList formats = mimeData->formats();
+	for (int i = 0; i < formats.size(); ++i) {
+#if defined(Q_OS_WIN)
+		// Visual Studio Line Cut/Copy markers
+		// If line cut/copy made by this application, see base name.
+		if (formats[i] == sVSEditorLineCutCopy)
+			return true;
+		// Otherwise see wrapped name.
+		if (formats[i] == sWrappedVSEditorLineCutCopy)
+			return true;
+#endif
+	}
+	return false;
+}
+
 bool ScintillaQt::ValidCodePage(int codePage) const
 {
 	return codePage == 0
@@ -224,6 +248,28 @@ bool ScintillaQt::ValidCodePage(int codePage) const
 	|| codePage == 1361;
 }
 
+std::string ScintillaQt::UTF8FromEncoded(std::string_view encoded) const {
+	if (IsUnicodeMode()) {
+		return std::string(encoded);
+	} else {
+		QTextCodec *codec = QTextCodec::codecForName(
+				CharacterSetID(CharacterSetOfDocument()));
+		QString text = codec->toUnicode(encoded.data(), static_cast<int>(encoded.length()));
+		return text.toStdString();
+	}
+}
+
+std::string ScintillaQt::EncodedFromUTF8(std::string_view utf8) const {
+	if (IsUnicodeMode()) {
+		return std::string(utf8);
+	} else {
+		QString text = QString::fromUtf8(utf8.data(), static_cast<int>(utf8.length()));
+		QTextCodec *codec = QTextCodec::codecForName(
+				CharacterSetID(CharacterSetOfDocument()));
+		QByteArray ba = codec->fromUnicode(text);
+		return std::string(ba.data(), ba.length());
+	}
+}
 
 void ScintillaQt::ScrollText(Sci::Line linesToMove)
 {
@@ -302,6 +348,10 @@ void ScintillaQt::CopyToModeClipboard(const SelectionText &selectedText, QClipbo
 		AddRectangularToMime(mimeData, su);
 	}
 
+	if (selectedText.lineCopy) {
+		AddLineCutCopyToMime(mimeData);
+	}
+
 	// Allow client code to add additional data (e.g rich text).
 	emit aboutToCopy(mimeData);
 
@@ -327,6 +377,7 @@ void ScintillaQt::PasteFromMode(QClipboard::Mode clipboardMode_)
 	QClipboard *clipboard = QApplication::clipboard();
 	const QMimeData *mimeData = clipboard->mimeData(clipboardMode_);
 	bool isRectangular = IsRectangularInMime(mimeData);
+	bool isLine = SelectionEmpty() && IsLineCutCopyInMime(mimeData);
 	QString text = clipboard->text(clipboardMode_);
 	QByteArray utext = BytesForDocument(text);
 	std::string dest(utext.constData(), utext.length());
@@ -334,9 +385,9 @@ void ScintillaQt::PasteFromMode(QClipboard::Mode clipboardMode_)
 	selText.Copy(dest, pdoc->dbcsCodePage, CharacterSetOfDocument(), isRectangular, false);
 
 	UndoGroup ug(pdoc);
-	ClearSelection(multiPasteMode == SC_MULTIPASTE_EACH);
+	ClearSelection(multiPasteMode == MultiPaste::Each);
 	InsertPasteShape(selText.Data(), selText.Length(),
-		selText.rectangular ? pasteRectangular : pasteStream);
+		isRectangular ? PasteShape::rectangular : (isLine ? PasteShape::line : PasteShape::stream));
 	EnsureCaretVisible();
 }
 
@@ -381,7 +432,7 @@ void ScintillaQt::NotifyFocus(bool focus)
 	Editor::NotifyFocus(focus);
 }
 
-void ScintillaQt::NotifyParent(SCNotification scn)
+void ScintillaQt::NotifyParent(NotificationData scn)
 {
 	scn.nmhdr.hwndFrom = wMain.GetID();
 	scn.nmhdr.idFrom = GetCtrlID();
@@ -390,8 +441,8 @@ void ScintillaQt::NotifyParent(SCNotification scn)
 
 void ScintillaQt::NotifyURIDropped(const char *uri)
 {
-	SCNotification scn = {};
-	scn.nmhdr.code = SCN_URIDROPPED;
+	NotificationData scn = {};
+	scn.nmhdr.code = Notification::URIDropped;
 	scn.text = uri;
 
 	NotifyParent(scn);
@@ -399,20 +450,20 @@ void ScintillaQt::NotifyURIDropped(const char *uri)
 
 bool ScintillaQt::FineTickerRunning(TickReason reason)
 {
-	return timers[reason] != 0;
+	return timers[static_cast<size_t>(reason)] != 0;
 }
 
 void ScintillaQt::FineTickerStart(TickReason reason, int millis, int /* tolerance */)
 {
 	FineTickerCancel(reason);
-	timers[reason] = startTimer(millis);
+	timers[static_cast<size_t>(reason)] = startTimer(millis);
 }
 
 // CancelTimers cleans up all fine-ticker timers and is non-virtual to avoid warnings when
 // called during destruction.
 void ScintillaQt::CancelTimers()
 {
-	for (TickReason tr = tickCaret; tr <= tickDwell; tr = static_cast<TickReason>(tr + 1)) {
+	for (size_t tr = static_cast<size_t>(TickReason::caret); tr <= static_cast<size_t>(TickReason::dwell); tr++) {
 		if (timers[tr]) {
 			killTimer(timers[tr]);
 			timers[tr] = 0;
@@ -422,9 +473,10 @@ void ScintillaQt::CancelTimers()
 
 void ScintillaQt::FineTickerCancel(TickReason reason)
 {
-	if (timers[reason]) {
-		killTimer(timers[reason]);
-		timers[reason] = 0;
+	const size_t reasonIndex = static_cast<size_t>(reason);
+	if (timers[reasonIndex]) {
+		killTimer(timers[reasonIndex]);
+		timers[reasonIndex] = 0;
 	}
 }
 
@@ -454,9 +506,9 @@ bool ScintillaQt::ChangeIdle(bool on)
 			idler.state = false;
 			qIdle = static_cast<QTimer *>(idler.idlerID);
 			qIdle->stop();
-			disconnect(qIdle, SIGNAL(timeout()), 0, 0);
+			disconnect(qIdle, SIGNAL(timeout()), nullptr, nullptr);
 			delete qIdle;
-			idler.idlerID = 0;
+			idler.idlerID = {};
 		}
 	}
 	return true;
@@ -467,7 +519,7 @@ bool ScintillaQt::SetIdle(bool on)
 	return ChangeIdle(on);
 }
 
-int ScintillaQt::CharacterSetOfDocument() const
+CharacterSet ScintillaQt::CharacterSetOfDocument() const
 {
 	return vs.styles[STYLE_DEFAULT].characterSet;
 }
@@ -526,46 +578,48 @@ public:
 	}
 };
 
-CaseFolder *ScintillaQt::CaseFolderForEncoding()
+std::unique_ptr<CaseFolder> ScintillaQt::CaseFolderForEncoding()
 {
 	if (pdoc->dbcsCodePage == SC_CP_UTF8) {
-		return new CaseFolderUnicode();
+		return std::make_unique<CaseFolderUnicode>();
 	} else {
 		const char *charSetBuffer = CharacterSetIDOfDocument();
 		if (charSetBuffer) {
 			if (pdoc->dbcsCodePage == 0) {
-				CaseFolderTable *pcf = new CaseFolderTable();
+				std::unique_ptr<CaseFolderTable> pcf = std::make_unique<CaseFolderTable>();
 				pcf->StandardASCII();
 				QTextCodec *codec = QTextCodec::codecForName(charSetBuffer);
 				// Only for single byte encodings
 				for (int i=0x80; i<0x100; i++) {
 					char sCharacter[2] = "A";
-					sCharacter[0] = i;
+					sCharacter[0] = static_cast<char>(i);
 					QString su = codec->toUnicode(sCharacter, 1);
 					QString suFolded = su.toCaseFolded();
-					QByteArray bytesFolded = codec->fromUnicode(suFolded);
-					if (bytesFolded.length() == 1) {
-						pcf->SetTranslation(sCharacter[0], bytesFolded[0]);
+					if (codec->canEncode(suFolded)) {
+						QByteArray bytesFolded = codec->fromUnicode(suFolded);
+						if (bytesFolded.length() == 1) {
+							pcf->SetTranslation(sCharacter[0], bytesFolded[0]);
+						}
 					}
 				}
 				return pcf;
 			} else {
-				return new CaseFolderDBCS(QTextCodec::codecForName(charSetBuffer));
+				return std::make_unique<CaseFolderDBCS>(QTextCodec::codecForName(charSetBuffer));
 			}
 		}
-		return 0;
+		return nullptr;
 	}
 }
 
-std::string ScintillaQt::CaseMapString(const std::string &s, int caseMapping)
+std::string ScintillaQt::CaseMapString(const std::string &s, CaseMapping caseMapping)
 {
-	if ((s.size() == 0) || (caseMapping == cmSame))
+	if ((s.size() == 0) || (caseMapping == CaseMapping::same))
 		return s;
 
 	if (IsUnicodeMode()) {
 		std::string retMapped(s.length() * maxExpansionCaseConversion, 0);
 		size_t lenMapped = CaseConvertString(&retMapped[0], retMapped.length(), s.c_str(), s.length(),
-			(caseMapping == cmUpper) ? CaseConversionUpper : CaseConversionLower);
+			(caseMapping == CaseMapping::upper) ? CaseConversion::upper : CaseConversion::lower);
 		retMapped.resize(lenMapped);
 		return retMapped;
 	}
@@ -573,7 +627,7 @@ std::string ScintillaQt::CaseMapString(const std::string &s, int caseMapping)
 	QTextCodec *codec = QTextCodec::codecForName(CharacterSetIDOfDocument());
 	QString text = codec->toUnicode(s.c_str(), static_cast<int>(s.length()));
 
-	if (caseMapping == cmUpper) {
+	if (caseMapping == CaseMapping::upper) {
 		text = text.toUpper();
 	} else {
 		text = text.toLower();
@@ -598,7 +652,7 @@ bool ScintillaQt::HaveMouseCapture()
 
 void ScintillaQt::StartDrag()
 {
-	inDragDrop = ddDragging;
+	inDragDrop = DragDrop::dragging;
 	dropWentOutside = true;
 	if (drag.Length()) {
 		QMimeData *mimeData = new QMimeData;
@@ -611,13 +665,13 @@ void ScintillaQt::StartDrag()
 		QDrag *dragon = new QDrag(scrollArea);
 		dragon->setMimeData(mimeData);
 
-		Qt::DropAction dropAction = dragon->exec(Qt::CopyAction|Qt::MoveAction);
+		Qt::DropAction dropAction = dragon->exec(static_cast<Qt::DropActions>(Qt::CopyAction|Qt::MoveAction));
 		if ((dropAction == Qt::MoveAction) && dropWentOutside) {
 			// Remove dragged out text
 			ClearSelection();
 		}
 	}
-	inDragDrop = ddNone;
+	inDragDrop = DragDrop::none;
 	SetDragPosition(SelectionPosition(Sci::invalidPosition));
 }
 
@@ -635,12 +689,10 @@ public:
 	void paintEvent(QPaintEvent *) override
 	{
 		if (pct->inCallTipMode) {
-			Surface *surfaceWindow = Surface::Allocate(0);
+			std::unique_ptr<Surface> surfaceWindow = Surface::Allocate(Technology::Default);
 			surfaceWindow->Init(this);
-			surfaceWindow->SetUnicodeMode(SC_CP_UTF8 == pct->codePage);
-			surfaceWindow->SetDBCSMode(pct->codePage);
-			pct->PaintCT(surfaceWindow);
-			delete surfaceWindow;
+			surfaceWindow->SetMode(SurfaceMode(pct->codePage, false));
+			pct->PaintCT(surfaceWindow.get());
 		}
 	}
 
@@ -680,37 +732,40 @@ void ScintillaQt::AddToPopUp(const char *label,
 	        this, SLOT(execCommand(QAction *)));
 }
 
-sptr_t ScintillaQt::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam)
+sptr_t ScintillaQt::WndProc(Message iMessage, uptr_t wParam, sptr_t lParam)
 {
 	try {
 		switch (iMessage) {
 
-		case SCI_SETIMEINTERACTION:
+		case Message::SetIMEInteraction:
 			// Only inline IME supported on Qt
 			break;
 
-		case SCI_GRABFOCUS:
+		case Message::GrabFocus:
 			scrollArea->setFocus(Qt::OtherFocusReason);
 			break;
 
-		case SCI_GETDIRECTFUNCTION:
+		case Message::GetDirectFunction:
 			return reinterpret_cast<sptr_t>(DirectFunction);
 
-		case SCI_GETDIRECTPOINTER:
+		case Message::GetDirectStatusFunction:
+			return reinterpret_cast<sptr_t>(DirectStatusFunction);
+
+		case Message::GetDirectPointer:
 			return reinterpret_cast<sptr_t>(this);
 
 		default:
 			return ScintillaBase::WndProc(iMessage, wParam, lParam);
 		}
 	} catch (std::bad_alloc &) {
-		errorStatus = SC_STATUS_BADALLOC;
+		errorStatus = Status::BadAlloc;
 	} catch (...) {
-		errorStatus = SC_STATUS_FAILURE;
+		errorStatus = Status::Failure;
 	}
 	return 0;
 }
 
-sptr_t ScintillaQt::DefWndProc(unsigned int, uptr_t, sptr_t)
+sptr_t ScintillaQt::DefWndProc(Message, uptr_t, sptr_t)
 {
 	return 0;
 }
@@ -718,7 +773,17 @@ sptr_t ScintillaQt::DefWndProc(unsigned int, uptr_t, sptr_t)
 sptr_t ScintillaQt::DirectFunction(
     sptr_t ptr, unsigned int iMessage, uptr_t wParam, sptr_t lParam)
 {
-	return reinterpret_cast<ScintillaQt *>(ptr)->WndProc(iMessage, wParam, lParam);
+	ScintillaQt *sci = reinterpret_cast<ScintillaQt *>(ptr);
+	return sci->WndProc(static_cast<Message>(iMessage), wParam, lParam);
+}
+
+sptr_t ScintillaQt::DirectStatusFunction(
+    sptr_t ptr, unsigned int iMessage, uptr_t wParam, sptr_t lParam, int *pStatus)
+{
+	ScintillaQt *sci = reinterpret_cast<ScintillaQt *>(ptr);
+	const sptr_t returnValue = sci->WndProc(static_cast<Message>(iMessage), wParam, lParam);
+	*pStatus = static_cast<int>(sci->errorStatus);
+	return returnValue;
 }
 
 // Additions to merge in Scientific Toolworks widget structure
@@ -726,7 +791,7 @@ sptr_t ScintillaQt::DirectFunction(
 void ScintillaQt::PartialPaint(const PRectangle &rect)
 {
 	rcPaint = rect;
-	paintState = painting;
+	paintState = PaintState::painting;
 	PRectangle rcClient = GetClientRectangle();
 	paintingAllText = rcPaint.Contains(rcClient);
 
@@ -734,11 +799,11 @@ void ScintillaQt::PartialPaint(const PRectangle &rect)
 	Paint(surfacePaint, rcPaint);
 	surfacePaint->Release();
 
-	if (paintState == paintAbandoned) {
+	if (paintState == PaintState::abandoned) {
 		// FIXME: Failure to paint the requested rectangle in each
 		// paint event causes flicker on some platforms (Mac?)
 		// Paint rect immediately.
-		paintState = painting;
+		paintState = PaintState::painting;
 		paintingAllText = true;
 
 		AutoSurface surface(this);
@@ -749,7 +814,7 @@ void ScintillaQt::PartialPaint(const PRectangle &rect)
 		scrollArea->viewport()->update();
 	}
 
-	paintState = notPainting;
+	paintState = PaintState::notPainting;
 }
 
 void ScintillaQt::DragEnter(const Point &point)
@@ -791,9 +856,9 @@ void ScintillaQt::DropUrls(const QMimeData *data)
 
 void ScintillaQt::timerEvent(QTimerEvent *event)
 {
-	for (TickReason tr=tickCaret; tr<=tickDwell; tr = static_cast<TickReason>(tr+1)) {
+	for (size_t tr=static_cast<size_t>(TickReason::caret); tr<=static_cast<size_t>(TickReason::dwell); tr++) {
 		if (timers[tr] == event->timerId()) {
-			TickFor(tr);
+			TickFor(static_cast<TickReason>(tr));
 		}
 	}
 }
