@@ -30,11 +30,11 @@
 # On Fedora 17, qmake is called qmake-qt4 so sepbuild.py should probe for correct name.
 # There are also problems with clang failing in the g++ 4.7 headers.
 
-# Turn off deprecation warnings if not interested:
-#export CXXFLAGS="-Wno-deprecated-declarations -D GLIB_DISABLE_DEPRECATION_WARNINGS"
+# Turn off glib deprecation warnings since gtk headers use deprecated items
+NO_GLIB_DEPRECATIONS="CXXFLAGS=-D GLIB_DISABLE_DEPRECATION_WARNINGS"
 
 # Run commands in parallel up to number of processors
-JOBS="-j $(getconf _NPROCESSORS_ONLN)"
+JOBS="--jobs=$(getconf _NPROCESSORS_ONLN)"
 
 cd ../..
 
@@ -43,16 +43,23 @@ cd ../..
 (
 cd scintilla/test/unit || exit
 make clean
-make $JOBS test
+make "$JOBS" test
 make clean
 )
 
 (
-cd scintilla/lexilla/src || exit
+cd lexilla/src || exit
 make clean
-make $JOBS
-cd ..
-cd test || exit
+make "$JOBS"
+)
+(
+cd lexilla/test || exit
+make clean
+make test
+make clean
+)
+(
+cd lexilla/test/unit || exit
 make clean
 make test
 make clean
@@ -61,13 +68,13 @@ make clean
 (
 cd scintilla/gtk || exit
 make clean
-make $JOBS
+make "$JOBS" "$NO_GLIB_DEPRECATIONS" GTK2=1
 )
 
 (
 cd scite/gtk || exit
 make clean
-make $JOBS
+make "$JOBS" "$NO_GLIB_DEPRECATIONS" GTK2=1
 )
 
 # ************************************************************
@@ -75,13 +82,13 @@ make $JOBS
 (
 cd scintilla/gtk || exit
 make clean
-make $JOBS GTK3=1
+make "$JOBS" "$NO_GLIB_DEPRECATIONS" GTK3=1
 )
 
 (
 cd scite/gtk || exit
 make clean
-make $JOBS GTK3=1
+make "$JOBS" GTK3=1
 )
 
 # ************************************************************
@@ -105,7 +112,7 @@ cd scintilla/qt || exit
 cd ScintillaEditBase || exit
 $QMAKENAME
 make clean
-make $JOBS
+make "$JOBS"
 make distclean
 )
 
@@ -114,35 +121,41 @@ cd ScintillaEdit || exit
 python3 WidgetGen.py
 $QMAKENAME
 make clean
-make $JOBS
+make "$JOBS"
 make distclean
 )
 
-(
-cd ScintillaEditPy || exit
-python2 sepbuild.py
-cd ../../test || exit
-python2 simpleTests.py
-python2 lexTests.py
-python2 performanceTests.py
-cd ../qt/ScintillaEditPy || exit
-python2 sepbuild.py --clean
-)
+#~ (
+#~ cd ScintillaEditPy || exit
+#~ python2 sepbuild.py
+#~ cd ../../test || exit
+#~ python2 simpleTests.py
+#~ python2 lexTests.py
+#~ python2 performanceTests.py
+#~ cd ../qt/ScintillaEditPy || exit
+#~ python2 sepbuild.py --clean
+#~ )
 
 )
 
 # ************************************************************
 # Target 4: clang build for GTK+ 2
+(
+cd lexilla/src || exit
+make clean
+make "$JOBS" CLANG=1 GTK2=1
+)
+
 (
 cd scintilla/gtk || exit
 make clean
-make $JOBS CLANG=1
+make "$JOBS" "$NO_GLIB_DEPRECATIONS" CLANG=1 GTK2=1
 )
 
 (
 cd scite/gtk || exit
 make clean
-make $JOBS CLANG=1
+make "$JOBS" "$NO_GLIB_DEPRECATIONS" CLANG=1 GTK2=1
 )
 
 # ************************************************************
@@ -150,27 +163,33 @@ make $JOBS CLANG=1
 (
 cd scintilla/gtk || exit
 make clean
-make $JOBS CLANG=1 GTK3=1
+make "$JOBS" "$NO_GLIB_DEPRECATIONS" CLANG=1 GTK3=1
 )
 
 (
 cd scite/gtk || exit
 make clean
-make $JOBS CLANG=1 GTK3=1
+make "$JOBS" CLANG=1 GTK3=1
 )
 
 # ************************************************************
 # Target 6: clang analyze for GTK+ 2
+(
+cd lexilla/src || exit
+make clean
+make "$JOBS" CLANG=1 analyze
+)
+
 (
 cd scintilla/gtk || exit
 make clean
-make $JOBS analyze
+make "$JOBS" "$NO_GLIB_DEPRECATIONS" analyze
 )
 
 (
 cd scite/gtk || exit
 make clean
-make $JOBS analyze
+make "$JOBS" "$NO_GLIB_DEPRECATIONS" analyze
 make clean
 cd ../..
 cd scintilla/gtk || exit
@@ -180,5 +199,6 @@ make clean
 # ************************************************************
 # Target 7: cppcheck static checker
 # Disabled as there are false warnings and some different style choices
-cppcheck --enable=all --max-configs=100 --suppressions-list=scintilla/cppcheck.suppress -I scintilla/src -I scintilla/include -I scintilla/lexlib -I scintilla/qt/ScintillaEditBase --template=gcc --quiet scintilla
+cppcheck --enable=all --max-configs=120 --suppressions-list=lexilla/cppcheck.suppress -I lexilla/include -I lexilla/lexlib --template=gcc --quiet lexilla
+cppcheck --enable=all --max-configs=100 --suppressions-list=scintilla/cppcheck.suppress -I scintilla/src -I scintilla/include -I scintilla/qt/ScintillaEditBase "-DSTDMETHODIMP_(type) type STDMETHODCALLTYPE" --template=gcc --quiet scintilla
 cppcheck --enable=all --max-configs=100 --suppressions-list=scite/cppcheck.suppress -I scite/src -I scintilla/include -I scite/lua/src -Ulua_assert -DPATH_MAX=260 --template=gcc --quiet scite
