@@ -16,16 +16,20 @@
 #include <string_view>
 #include <vector>
 #include <map>
+#include <set>
+#include <optional>
 #include <algorithm>
 #include <memory>
 
-#include "Platform.h"
-
+#include "ScintillaTypes.h"
 #include "ILoader.h"
 #include "ILexer.h"
-#include "Scintilla.h"
 
-#include "CharacterCategory.h"
+#include "Debugging.h"
+#include "Geometry.h"
+#include "Platform.h"
+
+#include "CharacterCategoryMap.h"
 
 #include "Position.h"
 #include "UniqueString.h"
@@ -34,7 +38,6 @@
 #include "RunStyles.h"
 #include "ContractionState.h"
 #include "CellBuffer.h"
-#include "KeyMap.h"
 #include "Indicator.h"
 #include "LineMarker.h"
 #include "Style.h"
@@ -49,6 +52,7 @@
 #include "EditModel.h"
 
 using namespace Scintilla;
+using namespace Scintilla::Internal;
 
 Caret::Caret() noexcept :
 	active(false), on(false), period(500) {}
@@ -60,33 +64,40 @@ EditModel::EditModel() : braces{} {
 	posDrag = SelectionPosition(Sci::invalidPosition);
 	braces[0] = Sci::invalidPosition;
 	braces[1] = Sci::invalidPosition;
-	bracesMatchStyle = STYLE_BRACEBAD;
+	bracesMatchStyle = StyleBraceBad;
 	highlightGuideColumn = 0;
+	hasFocus = false;
 	primarySelection = true;
-	imeInteraction = imeWindowed;
-	bidirectional = Bidirectional::bidiDisabled;
-	foldFlags = 0;
-	foldDisplayTextStyle = SC_FOLDDISPLAYTEXT_HIDDEN;
+	imeInteraction = IMEInteraction::Windowed;
+	bidirectional = Bidirectional::Disabled;
+	foldFlags = FoldFlag::None;
+	foldDisplayTextStyle = FoldDisplayTextStyle::Hidden;
 	hotspot = Range(Sci::invalidPosition);
+	hotspotSingleLine = true;
 	hoverIndicatorPos = Sci::invalidPosition;
 	wrapWidth = LineLayout::wrapWidthInfinite;
-	pdoc = new Document(SC_DOCUMENTOPTION_DEFAULT);
+	pdoc = new Document(DocumentOption::Default);
 	pdoc->AddRef();
 	pcs = ContractionStateCreate(pdoc->IsLarge());
 }
 
 EditModel::~EditModel() {
-	pdoc->Release();
+	try {
+		// This never throws but isn't marked noexcept for compatibility
+		pdoc->Release();
+	} catch (...) {
+		// Ignore any exception
+	}
 	pdoc = nullptr;
 }
 
 bool EditModel::BidirectionalEnabled() const noexcept {
-	return (bidirectional != Bidirectional::bidiDisabled) &&
-		(SC_CP_UTF8 == pdoc->dbcsCodePage);
+	return (bidirectional != Bidirectional::Disabled) &&
+		(CpUtf8 == pdoc->dbcsCodePage);
 }
 
 bool EditModel::BidirectionalR2L() const noexcept {
-	return bidirectional == Bidirectional::bidiR2L;
+	return bidirectional == Bidirectional::R2L;
 }
 
 void EditModel::SetDefaultFoldDisplayText(const char *text) {
@@ -98,10 +109,15 @@ const char *EditModel::GetDefaultFoldDisplayText() const noexcept {
 }
 
 const char *EditModel::GetFoldDisplayText(Sci::Line lineDoc) const noexcept {
-	if (foldDisplayTextStyle == SC_FOLDDISPLAYTEXT_HIDDEN || pcs->GetExpanded(lineDoc)) {
+	if (foldDisplayTextStyle == FoldDisplayTextStyle::Hidden || pcs->GetExpanded(lineDoc)) {
 		return nullptr;
 	}
 
 	const char *text = pcs->GetFoldDisplayText(lineDoc);
 	return text ? text : defaultFoldDisplayText.get();
 }
+
+InSelection EditModel::LineEndInSelection(Sci::Line lineDoc) const {
+	const Sci::Position posAfterLineEnd = pdoc->LineStart(lineDoc + 1);
+	return sel.InSelectionForEOL(posAfterLineEnd);
+}
