@@ -8,7 +8,7 @@
 #ifndef POSITIONCACHE_H
 #define POSITIONCACHE_H
 
-namespace Scintilla {
+namespace Scintilla::Internal {
 
 inline constexpr bool IsEOLChar(int ch) noexcept {
 	return (ch == '\r') || (ch == '\n');
@@ -38,15 +38,16 @@ public:
 // There are two points for some positions and this enumeration
 // can choose between the end of the first line or subline
 // and the start of the next line or subline.
-enum PointEnd {
-	peDefault = 0x0,
-	peLineEnd = 0x1,
-	peSubLineEnd = 0x2
+enum class PointEnd {
+	start = 0x0,
+	lineEnd = 0x1,
+	subLineEnd = 0x2,
+	endEither = lineEnd | subLineEnd,
 };
 
 class BidiData {
 public:
-	std::vector<FontAlias> stylesFonts;
+	std::vector<std::shared_ptr<Font>> stylesFonts;
 	std::vector<XYPOSITION> widthReprs;
 	void Resize(size_t maxLineLength_);
 };
@@ -60,7 +61,6 @@ private:
 	int lenLineStarts;
 	/// Drawing is only performed for @a maxLineLength characters on each line.
 	Sci::Line lineNumber;
-	bool inCache;
 public:
 	enum { wrapWidthInfinite = 0x7ffffff };
 
@@ -79,15 +79,12 @@ public:
 
 	std::unique_ptr<BidiData> bidiData;
 
-	// Hotspot support
-	Range hotspot;
-
 	// Wrapped line support
 	int widthLine;
 	int lines;
 	XYPOSITION wrapIndent; // In pixels
 
-	explicit LineLayout(int maxLineLength_);
+	LineLayout(Sci::Line lineNumber_, int maxLineLength_);
 	// Deleted so LineLayout objects can not be copied.
 	LineLayout(const LineLayout &) = delete;
 	LineLayout(LineLayout &&) = delete;
@@ -98,6 +95,8 @@ public:
 	void EnsureBidiData();
 	void Free() noexcept;
 	void Invalidate(ValidLevel validity_) noexcept;
+	Sci::Line LineNumber() const noexcept;
+	bool CanHold(Sci::Line lineDoc, int lineLength_) const noexcept;
 	int LineStart(int line) const noexcept;
 	int LineLength(int line) const noexcept;
 	enum class Scope { visibleOnly, includeEnd };
@@ -148,12 +147,13 @@ struct ScreenLine : public IScreenLine {
 /**
  */
 class LineLayoutCache {
-	int level;
-	std::vector<std::unique_ptr<LineLayout>>cache;
+public:
+private:
+	Scintilla::LineCache level;
+	std::vector<std::shared_ptr<LineLayout>>cache;
 	bool allInvalidated;
 	int styleClock;
-	int useCount;
-	void Allocate(size_t length_);
+	size_t EntryForLine(Sci::Line line) const noexcept;
 	void AllocateForLevel(Sci::Line linesOnScreen, Sci::Line linesInDoc);
 public:
 	LineLayoutCache();
@@ -164,18 +164,11 @@ public:
 	void operator=(LineLayoutCache &&) = delete;
 	virtual ~LineLayoutCache();
 	void Deallocate() noexcept;
-	enum {
-		llcNone=SC_CACHE_NONE,
-		llcCaret=SC_CACHE_CARET,
-		llcPage=SC_CACHE_PAGE,
-		llcDocument=SC_CACHE_DOCUMENT
-	};
 	void Invalidate(LineLayout::ValidLevel validity_) noexcept;
-	void SetLevel(int level_) noexcept;
-	int GetLevel() const noexcept { return level; }
-	LineLayout *Retrieve(Sci::Line lineNumber, Sci::Line lineCaret, int maxChars, int styleClock_,
+	void SetLevel(Scintilla::LineCache level_) noexcept;
+	Scintilla::LineCache GetLevel() const noexcept { return level; }
+	std::shared_ptr<LineLayout> Retrieve(Sci::Line lineNumber, Sci::Line lineCaret, int maxChars, int styleClock_,
 		Sci::Line linesOnScreen, Sci::Line linesInDoc);
-	void Dispose(LineLayout *ll) noexcept;
 };
 
 class PositionCacheEntry {
@@ -187,23 +180,27 @@ public:
 	PositionCacheEntry() noexcept;
 	// Copy constructor not currently used, but needed for being element in std::vector.
 	PositionCacheEntry(const PositionCacheEntry &);
+	PositionCacheEntry(PositionCacheEntry &&) noexcept = default;
 	// Deleted so PositionCacheEntry objects can not be assigned.
-	PositionCacheEntry(PositionCacheEntry &&) = delete;
 	void operator=(const PositionCacheEntry &) = delete;
 	void operator=(PositionCacheEntry &&) = delete;
 	~PositionCacheEntry();
-	void Set(unsigned int styleNumber_, const char *s_, unsigned int len_, const XYPOSITION *positions_, unsigned int clock_);
+	void Set(unsigned int styleNumber_, std::string_view sv, const XYPOSITION *positions_, unsigned int clock_);
 	void Clear() noexcept;
-	bool Retrieve(unsigned int styleNumber_, const char *s_, unsigned int len_, XYPOSITION *positions_) const noexcept;
-	static unsigned int Hash(unsigned int styleNumber_, const char *s, unsigned int len_) noexcept;
+	bool Retrieve(unsigned int styleNumber_, std::string_view sv, XYPOSITION *positions_) const noexcept;
+	static size_t Hash(unsigned int styleNumber_, std::string_view sv) noexcept;
 	bool NewerThan(const PositionCacheEntry &other) const noexcept;
 	void ResetClock() noexcept;
 };
 
 class Representation {
 public:
+	static constexpr size_t maxLength = 200;
 	std::string stringRep;
-	explicit Representation(const char *value="") : stringRep(value) {
+	RepresentationAppearance appearance;
+	ColourRGBA colour;
+	explicit Representation(std::string_view value="", RepresentationAppearance appearance_= RepresentationAppearance::Blob) :
+		stringRep(value), appearance(appearance_) {
 	}
 };
 
@@ -211,13 +208,22 @@ typedef std::map<unsigned int, Representation> MapRepresentation;
 
 class SpecialRepresentations {
 	MapRepresentation mapReprs;
-	short startByteHasReprs[0x100];
+	short startByteHasReprs[0x100] {};
+	bool crlf = false;
 public:
-	SpecialRepresentations();
-	void SetRepresentation(const char *charBytes, const char *value);
-	void ClearRepresentation(const char *charBytes);
-	const Representation *RepresentationFromCharacter(const char *charBytes, size_t len) const;
-	bool Contains(const char *charBytes, size_t len) const;
+	void SetRepresentation(std::string_view charBytes, std::string_view value);
+	void SetRepresentationAppearance(std::string_view charBytes, RepresentationAppearance appearance);
+	void SetRepresentationColour(std::string_view charBytes, ColourRGBA colour);
+	void ClearRepresentation(std::string_view charBytes);
+	const Representation *GetRepresentation(std::string_view charBytes) const;
+	const Representation *RepresentationFromCharacter(std::string_view charBytes) const;
+	bool Contains(std::string_view charBytes) const;
+	bool ContainsCrLf() const noexcept {
+		return crlf;
+	}
+	bool MayContain(unsigned char ch) const noexcept {
+		return startByteHasReprs[ch] != 0;
+	}
 	void Clear();
 };
 
@@ -254,7 +260,7 @@ public:
 	// Try to make each subdivided run lengthEachSubdivision or shorter.
 	enum { lengthEachSubdivision = 100 };
 	BreakFinder(const LineLayout *ll_, const Selection *psel, Range lineRange_, Sci::Position posLineStart_,
-		int xStart, bool breakForSelection, const Document *pdoc_, const SpecialRepresentations *preprs_, const ViewStyle *pvsDraw);
+		XYPOSITION xStart, bool breakForSelection, const Document *pdoc_, const SpecialRepresentations *preprs_, const ViewStyle *pvsDraw);
 	// Deleted so BreakFinder objects can not be copied.
 	BreakFinder(const BreakFinder &) = delete;
 	BreakFinder(BreakFinder &&) = delete;
@@ -271,17 +277,11 @@ class PositionCache {
 	bool allClear;
 public:
 	PositionCache();
-	// Deleted so PositionCache objects can not be copied.
-	PositionCache(const PositionCache &) = delete;
-	PositionCache(PositionCache &&) = delete;
-	void operator=(const PositionCache &) = delete;
-	void operator=(PositionCache &&) = delete;
-	~PositionCache();
 	void Clear() noexcept;
 	void SetSize(size_t size_);
-	size_t GetSize() const noexcept { return pces.size(); }
+	size_t GetSize() const noexcept;
 	void MeasureWidths(Surface *surface, const ViewStyle &vstyle, unsigned int styleNumber,
-		const char *s, unsigned int len, XYPOSITION *positions, const Document *pdoc);
+		std::string_view sv, XYPOSITION *positions);
 };
 
 }
