@@ -5,7 +5,7 @@
 // Copyright 1998-2003 by Neil Hodgson <neilh@scintilla.org>
 // The License.txt file describes the conditions under which this software may be distributed.
 
-#include <time.h>
+#include <ctime>
 
 #include "SciTEWin.h"
 #include "DLLFunction.h"
@@ -31,11 +31,7 @@
 const GUI::gui_char appName[] = GUI_TEXT("Sc1");
 #else
 const GUI::gui_char appName[] = GUI_TEXT("SciTE");
-#ifdef LOAD_SCINTILLA
 static const GUI::gui_char scintillaName[] = GUI_TEXT("Scintilla.DLL");
-#else
-static const GUI::gui_char scintillaName[] = GUI_TEXT("SciLexer.DLL");
-#endif
 #endif
 
 static GUI::gui_string GetErrorMessage(DWORD nRet) {
@@ -203,10 +199,12 @@ SciTEWin::SciTEWin(Extension *ext) : SciTEBase(ext) {
 	findStrip.SetSearcher(this);
 	replaceStrip.SetLocalizer(&localiser);
 	replaceStrip.SetSearcher(this);
+	filterStrip.SetLocalizer(&localiser);
+	filterStrip.SetSearcher(this);
 
 	flatterUI = UIShouldBeFlat();
 
-	appearance = CurrentAppearance();
+	appearance = WindowsAppearance();
 
 	cmdShow = 0;
 	heightBar = 7;
@@ -221,7 +219,6 @@ SciTEWin::SciTEWin(Extension *ext) : SciTEBase(ext) {
 	tooltipText[0] = '\0';
 	tbLarge = false;
 	modalParameters = false;
-	filterDefault = 1;
 	staticBuild = false;
 	menuSource = 0;
 
@@ -468,14 +465,14 @@ void SciTEWin::ReadEmbeddedProperties() {
 			const void *pv = ::LockResource(hmem);
 			if (pv) {
 				propsEmbed.ReadFromMemory(
-					static_cast<const char *>(pv), size, FilePath(), filter, NULL, 0);
+					static_cast<const char *>(pv), size, FilePath(), filter, nullptr, 0);
 			}
 		}
 		::FreeResource(handProps);
 	}
 }
 
-SystemAppearance SciTEWin::CurrentAppearance() const noexcept {
+SystemAppearance SciTEWin::WindowsAppearance() const noexcept {
 	SystemAppearance currentAppearance{};
 
 	HKEY hkeyPersonalize{};
@@ -499,7 +496,7 @@ SystemAppearance SciTEWin::CurrentAppearance() const noexcept {
 		;
 	const BOOL status = SystemParametersInfoW(SPI_GETHIGHCONTRAST, 0, &info, 0);
 	if (status) {
-		currentAppearance.highContrast = (info.dwFlags & HCF_HIGHCONTRASTON) ? 1 : 0;
+		currentAppearance.highContrast = (info.dwFlags & HCF_HIGHCONTRASTON) != 0;
 		if (currentAppearance.highContrast) {
 			// With high contrast, AppsUseLightTheme not correct so examine system background colour
 			const DWORD dwWindowColour = ::GetSysColor(COLOR_WINDOW);
@@ -510,6 +507,10 @@ SystemAppearance SciTEWin::CurrentAppearance() const noexcept {
 	return currentAppearance;
 }
 
+SystemAppearance SciTEWin::CurrentAppearance() const noexcept {
+	return WindowsAppearance();
+}
+
 void SciTEWin::ReadPropertiesInitial() {
 	SciTEBase::ReadPropertiesInitial();
 	if (tabMultiLine) {	// Windows specific!
@@ -608,7 +609,7 @@ void SciTEWin::ExecuteHelp(const char *cmd) {
 			typedef HWND (WINAPI *HelpFn)(HWND, const wchar_t *, UINT, DWORD_PTR);
 			HelpFn fnHHW = reinterpret_cast<HelpFn>(::GetProcAddress(hHH, "HtmlHelpW"));
 			if (fnHHW) {
-				XHH_AKLINK ak;
+				XHH_AKLINK ak {};
 				ak.cbStruct = sizeof(ak);
 				ak.fReserved = FALSE;
 				ak.pszKeywords = topic.c_str();
@@ -628,7 +629,7 @@ void SciTEWin::ExecuteHelp(const char *cmd) {
 }
 
 void SciTEWin::CopyAsRTF() {
-	const SA::Range cr = GetSelection();
+	const SA::Span cr = GetSelection();
 	std::ostringstream oss;
 	SaveToStreamRTF(oss, cr.start, cr.end);
 	const std::string rtf = oss.str();
@@ -857,22 +858,22 @@ void SciTEWin::ExecuteNext() {
 DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 	DWORD exitcode = 0;
 
-	if (jobToRun.jobType == jobShell) {
+	if (jobToRun.jobType == JobSubsystem::shell) {
 		ShellExec(jobToRun.command, jobToRun.directory.AsUTF8().c_str());
 		return exitcode;
 	}
 
-	if (jobToRun.jobType == jobHelp) {
+	if (jobToRun.jobType == JobSubsystem::help) {
 		ExecuteHelp(jobToRun.command.c_str());
 		return exitcode;
 	}
 
-	if (jobToRun.jobType == jobOtherHelp) {
+	if (jobToRun.jobType == JobSubsystem::otherHelp) {
 		ExecuteOtherHelp(jobToRun.command.c_str());
 		return exitcode;
 	}
 
-	if (jobToRun.jobType == jobGrep) {
+	if (jobToRun.jobType == JobSubsystem::grep) {
 		// jobToRun.command is "(w|~)(c|~)(d|~)(b|~)\0files\0text"
 		const char *grepCmd = jobToRun.command.c_str();
 		if (*grepCmd) {
@@ -936,7 +937,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 	STARTUPINFOW si = {};
 	si.cb = sizeof(STARTUPINFO);
 	si.dwFlags = STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES;
-	if (jobToRun.jobType == jobCLI)
+	if (jobToRun.jobType == JobSubsystem::cli)
 		si.wShowWindow = SW_HIDE;
 	else
 		si.wShowWindow = SW_SHOW;
@@ -964,7 +965,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 
 	const DWORD errCode = ::GetLastError();
 	// if jobCLI "System can't find" - try calling with command processor
-	if ((!running) && (jobToRun.jobType == jobCLI) && (
+	if ((!running) && (jobToRun.jobType == JobSubsystem::cli) && (
 				(errCode == ERROR_FILE_NOT_FOUND) || (errCode == ERROR_BAD_EXE_FORMAT))) {
 
 		std::string runComLine = "cmd.exe /c ";
@@ -1029,7 +1030,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 			std::vector<char> buffer(pipeBufferSize);
 
 			if (!::PeekNamedPipe(hPipeRead, &buffer[0],
-					     static_cast<DWORD>(buffer.size()), &bytesRead, &bytesAvail, NULL)) {
+					     static_cast<DWORD>(buffer.size()), &bytesRead, &bytesAvail, nullptr)) {
 				bytesAvail = 0;
 			}
 
@@ -1052,7 +1053,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 
 				const int bTest = ::WriteFile(hWriteSubProcess,
 							      jobToRun.input.c_str() + writingPosition,
-							      static_cast<DWORD>(bytesToWrite), &bytesWrote, NULL);
+							      static_cast<DWORD>(bytesToWrite), &bytesWrote, nullptr);
 
 				if (bTest) {
 					if ((writingPosition + bytesToWrite) / 1024 > writingPosition / 1024) {
@@ -1076,7 +1077,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 
 			} else if (bytesAvail > 0) {
 				const int bTest = ::ReadFile(hPipeRead, &buffer[0],
-							     static_cast<DWORD>(buffer.size()), &bytesRead, NULL);
+							     static_cast<DWORD>(buffer.size()), &bytesRead, nullptr);
 
 				if (bTest && bytesRead) {
 
@@ -1298,7 +1299,7 @@ void SciTEWin::Execute() {
 	if (scrollOutput)
 		wOutput.GotoPos(wOutput.Length());
 
-	if (jobQueue.jobQueue[cmdWorker.icmd].jobType == jobExtension) {
+	if (jobQueue.jobQueue[cmdWorker.icmd].jobType == JobSubsystem::extension) {
 		// Execute extensions synchronously
 		if (jobQueue.jobQueue[cmdWorker.icmd].flags & jobGroupUndo)
 			wEditor.BeginUndoAction();
@@ -1345,7 +1346,7 @@ void SciTEWin::StopExecute() {
 
 void SciTEWin::AddCommand(const std::string &cmd, const std::string &dir, JobSubsystem jobType, const std::string &input, int flags) {
 	if (cmd.length()) {
-		if ((jobType == jobShell) && ((flags & jobForceQueue) == 0)) {
+		if ((jobType == JobSubsystem::shell) && ((flags & jobForceQueue) == 0)) {
 			std::string pCmd = cmd;
 			parameterisedCommand = "";
 			if (pCmd[0] == '*') {
@@ -1382,7 +1383,7 @@ void SciTEWin::WorkerCommand(int cmd, Worker *pWorker) {
 
 void SciTEWin::QuitProgram() {
 	quitting = false;
-	if (SaveIfUnsureAll() != saveCancelled) {
+	if (SaveIfUnsureAll() != SaveResult::cancelled) {
 		if (fullScreen)	// Ensure tray visible on exit
 			FullScreenToggle();
 		quitting = true;
@@ -1468,7 +1469,7 @@ void SciTEWin::CreateUI() {
 	UIAvailable();
 }
 
-static bool IsSpaceOrTab(GUI::gui_char ch) noexcept {
+static constexpr bool IsSpaceOrTab(GUI::gui_char ch) noexcept {
 	return (ch == ' ') || (ch == '\t');
 }
 
@@ -1886,7 +1887,7 @@ LRESULT SciTEWin::KeyDown(WPARAM wParam) {
 	HMENU hMenu = ::GetMenu(MainHWND());
 	HMENU hToolsMenu = ::GetSubMenu(hMenu, menuTools);
 	for (int tool = 0; tool < toolMax; ++tool) {
-		MENUITEMINFO mii;
+		MENUITEMINFO mii {};
 		mii.cbSize = sizeof(MENUITEMINFO);
 		mii.fMask = MIIM_DATA;
 		if (::GetMenuItemInfo(hToolsMenu, IDM_TOOLS+tool, FALSE, &mii) && mii.dwItemData) {
@@ -1901,7 +1902,7 @@ LRESULT SciTEWin::KeyDown(WPARAM wParam) {
 	// exec it the command defined
 	for (const ShortcutItem &scut : shortCutItemList) {
 		if (KeyMatch(scut.menuKey, keyVal, modifierAsInt)) {
-			const int commandNum = SciTEBase::GetMenuCommandAsInt(scut.menuCommand.c_str());
+			const int commandNum = SciTEBase::GetMenuCommandAsInt(scut.menuCommand);
 			if (commandNum != -1) {
 				// its possible that the command is for scintilla directly
 				// all scintilla commands are larger then 2000
@@ -1965,19 +1966,18 @@ LRESULT SciTEWin::ContextMenuMessage(UINT iMessage, WPARAM wParam, LPARAM lParam
 	return 0;
 }
 
-void SciTEWin::CheckForScintillaFailure(SA::Status statusFailure) {
+void SciTEWin::CheckForScintillaFailure(SA::Status statusFailure) noexcept {
 	static int boxesVisible = 0;
 	if ((statusFailure > SA::Status::Ok) && (boxesVisible == 0)) {
 		boxesVisible++;
-		char buff[200] = "";
+		wchar_t buff[200] = L"";
 		if (statusFailure == SA::Status::BadAlloc) {
-			strcpy(buff, "Memory exhausted.");
+			wcscpy(buff, L"Memory exhausted.");
 		} else {
-			sprintf(buff, "Scintilla failed with status %d.", static_cast<int>(statusFailure));
+			swprintf(buff, std::size(buff), L"Scintilla failed with status %d.", static_cast<int>(statusFailure));
 		}
-		strcat(buff, " SciTE will now close.");
-		GUI::gui_string sMessage = GUI::StringFromUTF8(buff);
-		::MessageBox(MainHWND(), sMessage.c_str(), TEXT("Failure in Scintilla"), MB_OK | MB_ICONERROR | MB_APPLMODAL);
+		wcscat(buff, L" SciTE will now close.");
+		::MessageBox(MainHWND(), buff, TEXT("Failure in Scintilla"), MB_OK | MB_ICONERROR | MB_APPLMODAL);
 		exit(FALSE);
 	}
 }
@@ -2134,6 +2134,8 @@ LRESULT SciTEWin::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 					findStrip.Focus();
 				else if (replaceStrip.visible)
 					replaceStrip.Focus();
+				else if (filterStrip.visible)
+					filterStrip.Focus();
 				else if (userStrip.visible)
 					userStrip.Focus();
 				else
@@ -2265,7 +2267,7 @@ std::string SciTEWin::EncodeString(const std::string &s) {
 }
 
 // Convert String from doc encoding to UTF-8
-std::string SciTEWin::GetRangeInUIEncoding(GUI::ScintillaWindow &win, SA::Range range) {
+std::string SciTEWin::GetRangeInUIEncoding(GUI::ScintillaWindow &win, SA::Span range) {
 	std::string s = SciTEBase::GetRangeInUIEncoding(win, range);
 
 	UINT codePageDocument = wEditor.CodePage();
@@ -2273,14 +2275,13 @@ std::string SciTEWin::GetRangeInUIEncoding(GUI::ScintillaWindow &win, SA::Range
 	if (codePageDocument != SA::CpUtf8) {
 		codePageDocument = CodePageFromCharSet(characterSet, codePageDocument);
 		std::wstring sWide = StringDecode(std::string(s.c_str(), s.length()), codePageDocument);
-		std::string sMulti = StringEncode(sWide, CP_UTF8);
-		return std::string(sMulti.c_str(), 0, sMulti.length());
+		return StringEncode(sWide, CP_UTF8);
 	}
 	return s;
 }
 
 uintptr_t SciTEWin::EventLoop() {
-	MSG msg;
+	MSG msg {};
 	msg.wParam = 0;
 	BOOL going = TRUE;
 	while (going) {
@@ -2357,13 +2358,10 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR, int) {
 #endif
 
 	SciTEWin::Register(hInstance);
-	LexillaSetDefaultDirectory(GetSciTEPath(FilePath()).AsUTF8());
+	Lexilla::SetDefaultDirectory(GetSciTEPath(FilePath()).AsUTF8());
 #ifdef STATIC_BUILD
-	Scintilla_LinkLexers();
 	Scintilla_RegisterClasses(hInstance);
-	LexillaSetDefault([](const char *name) {
-		return CreateLexer(name);
-	});
+	Lexilla::SetDefault(CreateLexer);
 #else
 
 	HMODULE hmod = ::LoadLibrary(scintillaName);
@@ -2376,7 +2374,7 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR, int) {
 #endif
 
 	uintptr_t result = 0;
-	{
+	try {
 #ifdef NO_EXTENSIONS
 		Extension *extender = 0;
 #else
@@ -2398,11 +2396,18 @@ int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR, int) {
 			lptszCmdLine++;
 		try {
 			MainWind.Run(lptszCmdLine);
+			// Set the current directory always to the program directory
+			// to ensure a opened file directory is never locked.
+			GetSciTEPath(FilePath()).SetWorkingDirectory();
 			result = MainWind.EventLoop();
 		} catch (const SA::Failure &sf) {
 			MainWind.CheckForScintillaFailure(sf.status);
+		} catch (const std::bad_alloc &) {
+			::MessageBox({}, TEXT("Allocation failure"), TEXT("Failure in SciTE"), MB_OK | MB_ICONERROR | MB_APPLMODAL);
 		}
 		MainWind.Finalise();
+	} catch (std::bad_array_new_length &) {
+		::MessageBox({}, TEXT("Allocation failure"), TEXT("Failure to allocate SciTE at start up"), MB_OK | MB_ICONERROR | MB_APPLMODAL);
 	}
 
 #ifdef STATIC_BUILD
